
// Copyright (C) 2008 Indrek Mandre <indrek@mare.ee>
// http://www.mare.ee/indrek/flashme/

#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static void error (int n)
{
  fprintf (stderr, "error %d!\n", n);
  exit(1);
}

int main (int argc, char *argv[0])
{
  if ( argc != 3 )
      error(0);

  int fd = open (argv[1], O_RDWR | O_NOCTTY | O_SYNC);
  if ( fd == -1 )
      error(1);

  FILE *fin = fopen (argv[2], "rb");
  if ( !fin )
      error(2);

  // set up correct settings - 9600 8N1
  struct termios tos;
  tcgetattr (fd, &tos);

  tos.c_iflag = IGNBRK;
  tos.c_oflag = 0;
  tos.c_cflag = CLOCAL | CREAD | CS8 | HUPCL;
  tos.c_lflag = 0;
  tos.c_cc[VMIN] = 1;
  tos.c_cc[VTIME] = 5;

  cfsetospeed(&tos, B9600);
  cfsetispeed(&tos, B9600);

  tcsetattr(fd, TCSANOW, &tos);

  // signal AVR that we want to flash it
  write (fd, "F", 1);

  // send flashable binary size
  struct stat st;
  fstat (fileno (fin), &st);
  unsigned char len[2];
  len[0] = st.st_size & 0xff;
  len[1] = (st.st_size & 0xff00) >> 8;
  write (fd, len, 2);

  printf ("Flashing: ");
  fflush (stdout);

  int left = st.st_size;
  while (left)
    {
      unsigned char cmd;
      read (fd, &cmd, 1);
      // wait for next page request N <pagelen>
      if ( cmd != 'N' )
          continue;
      unsigned char len[2];
      read (fd, len, 2);
      int l = 0;
      l |= len[0];
      l |= len[1] << 8;
      printf (".");
      fflush (stdout);
      char buf[l];
      size_t r = fread (buf, 1, l, fin);
      write (fd, buf, r);
      left -= r;
    }

  printf (" done!\n");

  close (fd);
  fclose (fin);
  return 0;
}

